#ifndef _LINUX_VIRTIO_9P_H
#define _LINUX_VIRTIO_9P_H
#include <linux/virtio_config.h>

/* The ID for virtio console */
#define VIRTIO_ID_9P	9
/* Maximum number of virtio channels per partition (1 for now) */
#define MAX_9P_CHAN	1

#endif /* _LINUX_VIRTIO_9P_H */
