/*
 *  include/linux/timerfd.h
 *
 *  Copyright (C) 2007  Davide Libenzi <davidel@xmailserver.org>
 *
 */

#ifndef _LINUX_TIMERFD_H
#define _LINUX_TIMERFD_H


#define TFD_TIMER_ABSTIME (1 << 0)



#endif /* _LINUX_TIMERFD_H */

